@@ -1,5 +1,6 @@
 /*
  * codeconv.c
+ * Copyright(c) 2001 Takashi NEMOTO
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,6 +15,7 @@
  * Written by Takashi Nemoto (tnemoto@mvi.biglobe.ne.jp).
  * Modified by Kazuhiko <kazuhiko@ring.gr.jp>
  * Modified by Satomi <satomi@ring.gr.jp>
+ *
  */
 
 /* #define DEBUG_CODECONV */
@@ -76,8 +78,8 @@ static const char *eucjp_code_name = NULL;
 #endif
 
 /* Return code <0: error, -2: Output Buffer Overflow */
-static size_t current_to_euc PROTO((char **current, size_t *in_len,
-				   char **euc, size_t *out_len));
+size_t current_to_euc PROTO((char **current, size_t *in_len,
+			     char **euc, size_t *out_len));
 static size_t euc_to_current PROTO((char **euc, size_t *in_len,
 				   char **current, size_t *out_len));
 
@@ -103,6 +105,7 @@ static const char *shift_jis_names[] = {
     "SHIFT-JIS", "SHIFT_JIS", "SJIS", "CSSHIFTJIS", "SHIFTJIS", NULL
 };
 
+#ifdef HAVE_ICONV
 static const char *iso_2022_jp_names[] = {
   "ISO-2022-JP-3", "ISO-2022-JP-2", "ISO-2022-JP", 
   "CSISO2022JP", "CSISO2022JP2", "CSISO2022JP3", 
@@ -112,12 +115,13 @@ static const char *iso_2022_jp_names[] = {
 static const char *japanese_names[] = {
     "ja", "japanese", NULL
 };
+#endif
 
 static int match_str(str,str_list)
-     char *str;
-     char **str_list;
+     const char *str;
+     const char **str_list;
 {
-    char **ptr;
+    const char **ptr;
     for(ptr=str_list;*ptr!=NULL;ptr++) {
       if (strcasecmp(str,*ptr)==0) return 1;
     }
@@ -209,18 +213,18 @@ iconv_test(ctoe, etoc)
     olen=TEST_LENGTH;
 
     /* euc-jp => current code Ѵƥ */
-    if (iconv(etoc,&test1,&ilen,&test2,&olen) < 0)
+    if (iconv(etoc,&test1,&ilen,&test2,&olen) == ((size_t)-1))
         return 0;
-    if (iconv(etoc,NULL,&ilen,&test2,&olen) < 0)
+    if (iconv(etoc,NULL,&ilen,&test2,&olen) == ((size_t)-1))
         return 0;
 
     /* current code  뤫 */
     test2=test2_0;
     ilen=TEST_LENGTH-olen;
     olen=TEST_LENGTH;
-    if (iconv(ctoe,&test2,&ilen,&test3,&olen) < 0) 
+    if (iconv(ctoe,&test2,&ilen,&test3,&olen) == ((size_t)-1)) 
         return 0;
-    if (iconv(ctoe,NULL,&ilen,&test3,&olen) < 0) 
+    if (iconv(ctoe,NULL,&ilen,&test3,&olen) == ((size_t)-1))
         return 0;
 
     if (strncmp(test1_0,test3_0,strlen(test1_0)) != 0)
@@ -319,7 +323,7 @@ locale_init(encoding)
 	(conv_mode != IO_ICONV
 #ifdef HAVE_ICONV
 	 || (cur_to_euc != (iconv_t)-1 && euc_to_cur != (iconv_t)-1)
-#endif	/* HAVE_ICONV */
+#endif
 	 ))
 	return CODECONV_OK;
     initialized = 0;
@@ -389,6 +393,21 @@ xputs_raw(str, len, fp)
     return outlen;
 }
 
+/* Convert ISO8859-1 NO-BREAK SPACE to normal SPACE */
+/* NBSP(0xa0)  EUC-JP Ȥ֤ĤʤȦ */
+static int
+convert_nbsp(str, len)
+     char *str;
+     int len;
+{
+    while(len>0) {
+      if (((*str) & 0xff) == 0xa0) *str = 0x20;
+      str++;
+      len--;
+    }
+    return 1;
+}
+
 static int
 xputs2(str, len, fp)
      const char	*str;
@@ -424,6 +443,7 @@ xputs2(str, len, fp)
 	return EOF;
     }
     memcpy(buf1p, str, len);
+    convert_nbsp(buf1p, len);
     status=euc_to_current(&buf1p, &len1, &buf2p, &len2);
     if (status == -2) { /*  ΰ */
         buf1p = buf1p0;
@@ -442,7 +462,7 @@ xputs2(str, len, fp)
 	}
 	status=euc_to_current(&buf1p, &len1, &buf2p, &len2);
     } 
-    if (status < 0) {
+    if (status == CODECONV_ERROR || status == CODECONV_BUFFER_OVERFLOW) {
         /* Conversion Error   Τޤ޽ */
 #ifndef HAVE_ALLOCA
         free(buf1p0);
@@ -496,27 +516,46 @@ xvfprintf(fp, fmt, ap)
     return xputs2(buf1, len, fp);
 }
 
+/* USE_STDARG_H is defined in codeconv.h */
+#ifdef USE_STDARG_H
 int
-xfprintf(fp, fmt)
+xfprintf(FILE *fp, const char *fmt, ...)
+#else
+int
+xfprintf(fp, fmt, va_alist)
     FILE	*fp;
     const char	*fmt;
+    va_dcl
+#endif
 {
     int len;
     va_list ap;
-
+#ifdef USE_STDARG_H
     va_start(ap, fmt);
+#else
+    va_start(ap);
+#endif
     len = xvfprintf(fp, fmt, ap);
     va_end(ap);
     return len;
 }
 
 int
-xprintf(fmt)
+#ifdef USE_STDARG_H
+xprintf(const char *fmt, ...)
+#else
+xprintf(fmt, va_alist)
     const char *fmt;
+    va_dcl
+#endif
 {
     int len;
     va_list ap;
+#ifdef USE_STDARG_H
     va_start(ap, fmt);
+#else
+    va_start(ap);
+#endif
     len = xvfprintf(stdout, fmt, ap);
     va_end(ap);
     return len;
@@ -561,7 +600,7 @@ xfgets(str, size, fp)
 #ifndef HAVE_ALLOCA
     free(ibuf0);
 #endif
-    if (status >=0 || status == -2) return str0;
+    if (status != CODECONV_ERROR) return str0;
     return NULL;
 }
 
@@ -613,20 +652,20 @@ size_t current_to_euc (in_buf,in_len,out_buf,out_len)
      char **in_buf, **out_buf;
      size_t *in_len,*out_len;
 {
-    size_t ret;
     static int output_left = -1;
     int c1, c2;
     size_t count = 0;
 
 #ifdef HAVE_ICONV
     if (conv_mode == IO_ICONV) {
+        size_t ret;
         if (cur_to_euc == (iconv_t) -1)
 	    return CODECONV_ERROR;
         ret = iconv(cur_to_euc,in_buf,in_len,out_buf,out_len);
-	if (ret >= 0) 
+	if (ret != ((size_t)-1)) 
 	    ret = iconv(cur_to_euc, NULL, in_len, out_buf, out_len);
 #if defined (HAVE_ERRNO_H) && defined (E2BIG)
-	if (ret < 0) {
+	if (ret == ((size_t)-1)) {
 	    if (errno == E2BIG) 
 	        return CODECONV_BUFFER_OVERFLOW;
 	    return CODECONV_ERROR;
@@ -722,20 +761,20 @@ size_t euc_to_current (in_buf,in_len,out_buf,out_len)
      char **in_buf, **out_buf;
      size_t *in_len,*out_len;
 {
-    size_t ret;
     static int output_left = -1;
     int c1, c2;
     size_t count = 0;
 
 #ifdef HAVE_ICONV
     if (conv_mode == IO_ICONV) {
+        size_t ret;
         if (euc_to_cur == (iconv_t) -1)
 	    return CODECONV_ERROR;
         ret = iconv(euc_to_cur,in_buf,in_len,out_buf,out_len);
-        if (ret >= 0)
+        if (ret != ((size_t)-1))
 	    ret = iconv(euc_to_cur,NULL,in_len,out_buf,out_len);
 #if defined (HAVE_ERRNO_H) && defined (E2BIG)
-	if (ret < 0) {
+	if (ret == ((size_t)-1)) {
 	    if (errno == E2BIG) 
 	        return CODECONV_BUFFER_OVERFLOW;
 	    return CODECONV_ERROR;
